/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import java.util.Map;

import edu.mit.eecs.parserlib.UnableToParseException;


/**
 * 
 * This class represents a string constant in the grammar.
 * The type parameter <code>NT</code> is an Enum type that represents the set of nonterminals available in the grammar.
 * @author asolar
 *
 * @param <NT>
 */
public class Constant<NT extends Enum<NT>> extends GrammarTerm<NT> {
    
    private final String contents;
    
    /**
     * Create a constant from a string.
     * @param contents  May be an empty string
     */
    public Constant(String contents) {
        super();
        this.contents = contents;
    }

    
    @Override
    public ParseResult<NT> parse(String s, int pos, Map<NT, GrammarTerm<NT>> definitions, ParserState<NT> state) throws UnableToParseException {
        int newpos = pos + contents.length();
        if(newpos > s.length()){
            return new ParseResult<NT>(state.current(), contents, pos);
        }
        String l = s.substring(pos, newpos);
        if(l.equals(contents)){
            return new ParseResult<NT>(new InternalParseTree<NT>(this.contents, pos), newpos);
        }
        return new ParseResult<NT>(state.current(), contents, pos);
    }
    
    @Override
    public String toString(){
        return "'" + contents + "'";
    }
}
