/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import java.util.Map;

import edu.mit.eecs.parserlib.UnableToParseException;


/**
 * This class represents a grammar term that throws a parse exception if its child fails to match, rather than
 * backtracking.
 * 
 * @param <NT> an Enum type that represents the set of nonterminals available in the grammar.
 */
public class FailFast<NT extends Enum<NT>> extends GrammarTerm<NT> {
    private final GrammarTerm<NT> child;
    
    
    /**
     * Make a FailFast enclosing a grammar term.
     * @param child term that should fail fast
     */
    public FailFast(GrammarTerm<NT> child) {
        this.child = child;
    }
    
    @Override
    public ParseResult<NT> parse(String s, int pos, Map<NT, GrammarTerm<NT>> definitions, ParserState<NT> state) throws UnableToParseException{
        ParseResult<NT> pr = child.parse(s, pos, definitions, state);
        if (pr.failed()) throw new ParseException("", pr.nonterminal.toString(), pr.expectedText, "", pr.pos);
        return pr;
    }
    
    @Override
    public String toString(){
        return "failfast(" + child + ")";
    }

}
