/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
/* Copyright (c) 2016-2017 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import java.util.Map;

import edu.mit.eecs.parserlib.UnableToParseException;


/**
 * This abstract class defines a term in a grammar. 
 * The type parameter <code>NT</code> is an Enum type that represents the set of nonterminals available in the grammar.
 * Every GrammarTerm must implement a parse method that parses that particular kind of term.
 * @author asolar
 *
 * @param <NT>
 */
public abstract class GrammarTerm<NT extends Enum<NT>> {
    
    /**
     * Given a string s and a position pos within the string, this method will attempt to parse 
     * this particular GrammarTerm from the string. If the GrammarTerm is successfully parsed, 
     * the ParseResult will include a {@link InternalParseTree} for the term and the position in the string where
     * that term ends. This means that {@link ParseResult}.pos is between pos and s.length.
     * 
     * If the GrammarTerm is not parsed successfully, then the ParseResult will
     * indicate that (see definition of {@link ParseResult}). 
     *  
     * 
     * @param s
     * @param pos should be a valid position in the string s.
     * @param definitions should contain a definition of *all* the nonterminals in the grammar.
     * @return result of matching this grammar term against s starting from position pos
     * @throws UnableToParseException if a parse failure occurs that can't be fixed by backtracking
     */
    public abstract ParseResult<NT> parse(String s, int pos, Map<NT, GrammarTerm<NT>> definitions, ParserState<NT> state) throws UnableToParseException;
}
