/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import java.util.Map;
import java.util.regex.Pattern;

import edu.mit.eecs.parserlib.UnableToParseException;

/**
 * Matches exactly one character described by a regular expression, which is typically one of:
 * <pre>
 *    .     any character (including line terminators)
 *    [...] character range
 *    \d, \s, \w   character class   
 */
public class OneCharacterRegex<NT extends Enum<NT>> extends GrammarTerm<NT> {
    private final Pattern regex;

    public OneCharacterRegex(String regex) {
        super();
        this.regex = Pattern.compile(regex, Pattern.DOTALL);
    }
    
    
    @Override
    public ParseResult<NT> parse(String s, int pos, Map<NT, GrammarTerm<NT>> definitions, ParserState<NT> state) throws UnableToParseException {
        if (pos >= s.length()) {
            return new ParseResult<NT>(state.current(), toString(), pos);
        }
        String l = s.substring(pos, pos+1);
        if (regex.matcher(l).matches()) {
            return new ParseResult<NT>(new InternalParseTree<NT>(l, pos), pos+1);
        }
        return new ParseResult<NT>(state.current(), toString(), pos);
    }
    
    @Override
    public String toString() {
        return regex.toString();
    }
}
