/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import edu.mit.eecs.parserlib.UnableToParseException;

/**
 * ParseException represents a failure to match a string against a grammar, that can't be fixed
 * by backtracking.
 */

public class ParseException extends UnableToParseException {
    private static final long serialVersionUID = -1721830847163080754L;
    
    public final String nonterminalName;
    // nonterminalName is String rather than NT because Java doesn't allow a generic
    // class like ParseException<NT> to subclass an unparameterized class like RuntimeException
    
    public final String expectedText;
    public final String textBeingParsed;
    public final int pos;
    
    public ParseException(String message, String nonterminalName, String expectedText, String textBeingParsed, int pos) {
        super(Display.makeErrorMessage(message, nonterminalName, expectedText, textBeingParsed, pos, "string being parsed"));
        this.nonterminalName = nonterminalName;
        this.expectedText = expectedText;
        this.textBeingParsed = textBeingParsed;
        this.pos = pos;
    }
        
}
