/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import edu.mit.eecs.parserlib.UnableToParseException;

/**
 * Class keeps class of state during parsing
 * @author asolar
 *
 */
public class ParserState<NT> {

    private Stack<NT> stack = new Stack<>();
    private Map<NT, Stack<Integer> > first = new HashMap<>();
    
    public void enter(int pos, NT nonterminal) throws UnableToParseException {
        if(first.containsKey(nonterminal)) {
            Stack<Integer> s = first.get(nonterminal); 
            if(!s.isEmpty() && s.peek() ==pos){
                throw new GrammarException("detected left recursion in rule for " + nonterminal.toString().toUpperCase());
            }
            s.push(pos);
        }else{
            Stack<Integer> s = new Stack<>();
            s.push(pos);
            first.put(nonterminal, s);
        }
        stack.push(nonterminal);
    }
    public void leave(NT nonterminal){
        assert first.containsKey(nonterminal);
        first.get(nonterminal).pop();
        
        NT last = stack.pop();
        assert last == nonterminal;        
    }
    
    public NT current() {
        return stack.peek();
    }
}
