/* Copyright (c) 2016-2020 MIT 6.005/6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */
package edu.mit.eecs.parserlib.internal;

import java.util.Map;

import edu.mit.eecs.parserlib.UnableToParseException;


/**
 * Represents an {@literal @}skip occurrence.
 * 
 * The type parameter <code>NT</code> is an Enum type that represents the set of nonterminals available in the grammar.
 *
 * @param <NT>
 */
public class Skip<NT extends Enum<NT>> extends Repetition<NT> {
    
    /**
     * Construct a skip of one occurrence of body.
     * @param body of this skip
     */
    public Skip(GrammarTerm<NT> body) {
        super(body, HowMany.ZERO_OR_MORE);
    }
    
    @Override
    public ParseResult<NT> parse(String s, int pos, Map<NT, GrammarTerm<NT>> definitions, ParserState<NT> state) throws UnableToParseException {
        ParseResult<NT> pr = super.parse(s, pos, definitions, state);
        if (pr.failed()) {
            pr = new ParseResult<NT>(new InternalParseTree<NT>("", pos), pos);
        }
        pr.tree.markAsSkipped();
        return pr;      
    }

    @Override
    public String toString(){
        return "(?<skip>" + super.toString() + ")";
    }
    
}
